#!/usr/bin/env python
#
# grib_checker.py checks Checks whatever you want to check in your life
#
# AUTHOR: BSC-CNS I. CERNICHARO
#

import sys
import numpy as np
from eccodes import GribFile

import data_checks
import metadata_checks

class grib_content:
    """grib_content object class set a list of checks and properties where
    the content of a grib file will be evaluated. All it needs to initiate
    is the file path of the GRIB file we will evaluate
    """
    def __init__(self,filename):
        self.name = filename
        self.grib_content = GribFile(self.name)
        self.pname = filename.split("/")[-1]
        self.log = ""

    def print_num_keys(self):
        # It returns the file name without the path
        HEAD = "#NUM MSSGs: "
        if len(HEAD) != 24: 
            HEAD += (24-len(HEAD))*" "
        mssg = HEAD +str(len(self.grib_content))+" messages"
        return mssg

    def check_data_coords(self):
        # The following lists will be filled with the log report for each
        # message. Coordinates and data values will be tested
        coords_log = list()
        data_log = list()
        index = list()

        # Checking all messages. Here we call methods that are imported from
        # data_checks submodule
        for i,msg in enumerate(self.grib_content):
            coords_log.append(data_checks.check_coords(msg))
            data_log.append(data_checks.check_data_values(msg))
            index.append(i)
        
        err_coords = coords_log.count("ERROR")
        err_data = data_log.count("ERROR")

        if err_coords != 0:
            HEAD = "#V_COORDS >> WARNING: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_coord = "\n" + HEAD + "One or more messages have incorrect data"
        else:
            HEAD = "#V_COORDS >> INFO: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_coord = "\n" + HEAD + "Everything OK with coordinates"

        if err_data != 0:
            HEAD = "#V_DATA >> WARNING: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_data = "\n" + HEAD + "One or more messages have incorrect data"
        else:
            HEAD = "#V_DATA >> INFO: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_data = "\n" + HEAD + "Everything OK with data values"

        return msg_data,msg_coord

    def check_meta_grid(self):
        # The following lists will be filled with the log report for each
        # message. Metadata of the header and of the grid description section
        # will be tested.
        meta_log = list()
        grid_log = list()
        index = list()
        # These lists, g_ed and grid_t will check the homogeneity of the 
        # messages for the GRIBEditionNumber and gridType keys.
        g_ed = list()
        grid_t = list()
        missing_meta = list()
        missing_grid = list()

        # Checking all messages. Here we call methods that are imported from
        # metadata_checks submodule
        for i,msg in enumerate(self.grib_content):
            index.append(i)

            g_ed.append(msg["GRIBEditionNumber"])
            grid_t.append(msg["gridType"])

            meta_cck = metadata_checks.check_metada_keys(msg)
            grid_cck = metadata_checks.check_grid_keys(msg)

            if len(meta_cck) > 1:
                for item in meta_cck[1]:
                    if item not in missing_meta:
                        missing_meta.append(item)

            if len(grid_cck) > 1:
                for item in grid_cck[1]:
                    if item not in missing_grid:
                        missing_grid.append(item)

            meta_log.append(meta_cck[0])
            grid_log.append(grid_cck[0])

        grib_editions = list(set(g_ed))
        grid_types = list(set(grid_t))
        
        err_meta = meta_log.count("ERROR")
        err_grid = grid_log.count("ERROR")

        if len(grib_editions) > 1:
            HEAD = "#FK-GRIB_ED >> WARNING: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_editions = "\n" + HEAD + "Grib Edition Check:"
            msg_editions += "\n" + HEAD + "tFound different Grib editions in the messages"
            for item in grib_editions:
                msg_editions += "\n" + HEAD + "\t\tEdition: " + str(item)
        else:
            HEAD = "#FK-GRIB_ED >> INFO: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_editions = "\n" + HEAD + "Grib Edition Check:"
            msg_editions += "\n" + HEAD + "\tEverything OK, one unique edition found"
            msg_editions += "\n" + HEAD + "\t\tEdition: " + str(grib_editions[0])

        if len(grid_types) > 1:
            HEAD = "#FK-GRIB_TYP >> WARNING: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_grid_type = "\n" + HEAD + "Grid Type Check:"
            msg_grid_type += "\n" + HEAD + "\tFound different Grid Type in the messages"
            for item in grid_types:
                msg_grid_type += "\n" + HEAD + "\t\tGrid type: " + str(item)
        else:
            HEAD = "#FK-GRIB_TYP >> INFO: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_grid_type = "\n" + HEAD + "Grid Type Check:"
            msg_grid_type += "\n" + HEAD + "\tEverything OK, one unique Grid Type found"
            msg_grid_type += "\n" + HEAD + "\t\tGrid type: " + str(grid_types[0])

        if err_meta != 0:
            HEAD = "#MD-MSS >> WARNING: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_meta = "\n" + HEAD + "Metadata Check:"
            msg_meta += "\n" + HEAD + "\tOne or more messages have missing KEYS"
            for item in missing_meta:
                msg_meta += "\n" + HEAD + "\t\tKey: " + str(item)
        else:
            HEAD = "#MD-MSS >> INFO: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_meta = "\n" + HEAD + "Metadata Check:"
            msg_meta += "\n" + HEAD + "tEverything OK"

        if err_grid != 0:
            HEAD = "#MD-GRID >> WARNING: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_grid = "\n" + HEAD + "Grid Metada Check:"
            msg_grid += "\n" + HEAD + "\tOne or more messages have missing Keys"
            for item in missing_grid:
                msg_grid += "\n" + HEAD + "\t\tKey: " + str(item)
        else:
            HEAD = "#MD-GRID >> INFO: "
            if len(HEAD) != 24: 
                HEAD += (24-len(HEAD))*" "
            msg_grid = "\n" + HEAD + "Grid Metada Check:"
            msg_grid += "\n" + HEAD + "\tEverything OK with data values"

        return msg_meta,msg_grid,msg_grid_type,msg_editions

        # STILL WORKING ON THIS
